#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <glob.h>
#include "pvo-block.hh"
#include "pvo-header.hh"
#include "pvo-FITS.hh"
#include "pvo-utils.hh"

using namespace PVO;
using namespace PVO::Header;
using namespace PVO::Block;
using namespace PVO::FITS;
using std::string;


const std::string help_text = 
"Usage:\n\
  1. pvosummary [FILE]\n\
  2. pvosummary\n\
  3. pvosummary -h\n\
---------\n\
  1. Prints RA, DEC, BEAM FWHM, DATE-OBS for single FITS file.\n\
  2. Prints RA, DEC, BEAM FWHM, DATE-OBS for all FITS files in the current directory.\n\
  3. Prints this help information.\n\
 Note that if a keyword is not present, --- is printed.\n\
 FITS files are assumed to have suffixes of .fits or .rf\n";

int main(int argc, char * argv[])
{

  bool verboseflag = false;

  std::vector<std::string> namelist;
  unsigned int i;
  switch(argc){
  case 1:
    glob_t globbuf1,globbuf2;
    globbuf1.gl_offs = 0;
    globbuf2.gl_offs = 0;
    glob("*.fits", GLOB_DOOFFS, NULL, &globbuf1);
    glob("*.rf", GLOB_DOOFFS, NULL, &globbuf2);
    namelist.resize(globbuf1.gl_pathc+globbuf2.gl_pathc);
    for(i=0;i<globbuf1.gl_pathc+globbuf1.gl_offs;i++) 
      namelist[i] = globbuf1.gl_pathv[i];
    for(i=0;i<globbuf2.gl_pathc+globbuf2.gl_offs;i++) 
      namelist[globbuf1.gl_pathc+i] = globbuf2.gl_pathv[i];
    break;
  case 2:
    if(strcmp(argv[1],"-h")==0){
      std::cout << help_text;
      return 0;
    }
    namelist.resize(1);
    namelist[0] = argv[1];
    break;
  default:
    std::cerr << help_text;
    return 0;
    break;
  }

  int maxsize = 8; // size of "FILENAME"
  for(i=0;i<namelist.size();i++) 
    maxsize = std::max(maxsize,int(namelist[i].size()));

  std::cout << std::setfill('-')  
	    << std::setw(maxsize+1+15+15+31+11) << "-"
	    << "\n" << std::setfill(' ') 
	    << std::setw(maxsize) << "FILENAME"  << ": "
	    << std::setw(14)      << "RA"        << " "
	    << std::setw(14)      << "DEC"       << " "
	    << std::setw(10)      << "BEAM FWHM" << " "
	    << std::setw(30)      << "DATE-OBS"  << "\n"
	    << std::setw(maxsize) << ""          << "  "
	    << std::setw(14)      << ""          << " "
	    << std::setw(14)      << ""          << " "
	    << std::setw(10)      << "[deg]"     << " "
	    << std::setw(30)      << ""          << "\n"
	    << std::setfill('-')  
	    << std::setw(maxsize+1+15+15+31+11) << "-"
	    << "\n"<< std::setfill(' ') ;

  for(i=0;i<namelist.size();i++){

    PSRFits fitsfile;
    fitsfile.setVerbosity(verboseflag);
    int status = fitsfile.openFile(namelist[i]);
  
    if(status==0){
      fitsfile.gotoPHU();
    
      std::string ra,dec,date;
      std::string missing = "---";
      if(fitsfile.getKeyword<std::string>("RA",ra)!=0) ra = missing;
      if(fitsfile.getKeyword<std::string>("DEC",dec)!=0) dec = missing;
      if(fitsfile.getKeyword<std::string>("DATE-OBS",date)!=0) date = missing;
      float beamradius,bmaj,bmin;
      fitsfile.getKeyword<float>("BMAJ",bmaj);
      fitsfile.getKeyword<float>("BMIN",bmin);
      beamradius = 0.5*(bmaj+bmin); // average FWHM, in deg
    
      std::cout << std::setw(maxsize) << namelist[i]  << ": "
		<< std::setw(14)      << ra           << " "
		<< std::setw(14)      << dec          << " "
		<< std::setw(10)      << beamradius   << " "
		<< std::setw(30)      << date         << "\n";
    }
  }

}
